@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2012-2019, VU University Amsterdam
+    Copyright (c)  2012-2021, VU University Amsterdam
                               CWI, Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -63,6 +64,12 @@
 :- use_module(library(http/json)).
 :- use_module(library(http/http_client), []).   % plugin for POST support
 :- use_module(library(prolog_config)).
+:- use_module(library(debug), [assertion/1]).
+:- use_module(library(pairs), [group_pairs_by_key/2, pairs_values/2]).
+% Stuff we may not have and may not need
+:- autoload(library(git)).
+:- autoload(library(sgml)).
+:- autoload(library(sha)).
 
 /** <module> A package manager for Prolog
 
@@ -100,11 +107,15 @@ makes installed packages available as libraries.
                  *******************************/
 
 %!  current_pack(?Pack) is nondet.
+%!  current_pack(?Pack, ?Dir) is nondet.
 %
 %   True if Pack is a currently installed pack.
 
 current_pack(Pack) :-
-    '$pack':pack(Pack, _).
+    current_pack(Pack, _).
+
+current_pack(Pack, Dir) :-
+    '$pack':pack(Pack, Dir).
 
 %!  pack_list_installed is det.
 %
@@ -283,7 +294,7 @@ pack_info_term(title(atom)).
 pack_info_term(keywords(list(atom))).
 pack_info_term(description(list(atom))).
 pack_info_term(version(version)).
-pack_info_term(author(atom, email_or_url)).     % Persons
+pack_info_term(author(atom, email_or_url_or_empty)).     % Persons
 pack_info_term(maintainer(atom, email_or_url)).
 pack_info_term(packager(atom, email_or_url)).
 pack_info_term(home(atom)).                     % Home page
@@ -306,6 +317,11 @@ error:has_type(email_or_url, Address) :-
     ->  true
     ;   uri_is_global(Address)
     ).
+error:has_type(email_or_url_or_empty, Address) :-
+    (   Address == ''
+    ->  true
+    ;   error:has_type(email_or_url, Address)
+    ).
 error:has_type(dependency, Value) :-
     is_dependency(Value, _Token, _Version).
 
@@ -447,7 +463,7 @@ search_info(download(_)).
 %       star (*) for the version.  In this case pack_install asks
 %       for the directory content and selects the latest version.
 %     * GIT URL (not well supported yet)
-%     * A local directory name given as =|file://|= URL.
+%     * A local directory name given as =|file://|= URL or `'.'`
 %     * A package name.  This queries the package repository
 %       at http://www.swi-prolog.org
 %
@@ -505,6 +521,19 @@ pack_default_options(FileURL, Pack, _, Options) :-      % Install from directory
         Options = [url(DirURL), version(Version)]
     ;   throw(error(existence_error(key, version, Dir),_))
     ).
+pack_default_options('.', Pack, _, Options) :-          % Install from CWD
+    pack_info_term('.', name(Pack)),
+    !,
+    working_directory(Dir, Dir),
+    (   pack_info_term(Dir, version(Version))
+    ->  uri_file_name(DirURL, Dir),
+        Options = [url(DirURL), version(Version) | Options1],
+        (   current_prolog_flag(windows, true)
+        ->  Options1 = []
+        ;   Options1 = [link(true), rebuild(make)]
+        )
+    ;   throw(error(existence_error(key, version, Dir),_))
+    ).
 pack_default_options(URL, Pack, _, Options) :-          % Install from URL
     pack_version_file(Pack, Version, URL),
     download_url(URL),
@@ -602,9 +631,19 @@ url_menu_item(URL, URL=install_from(URL)).
 %     * upgrade(+Boolean)
 %     If `true` (default `false`), upgrade package if it is already
 %     installed.
+%     * rebuild(Condition)
+%     Rebuild the foreign components.  Condition is one of
+%     `if_absent` (default, do nothing if the directory with foreign
+%     resources exists), `make` (run `make`) or `true` (run `make
+%     distclean` followed by the default configure and build steps).
 %     * git(+Boolean)
 %     If `true` (default `false` unless `URL` ends with =.git=),
 %     assume the URL is a GIT repository.
+%     * link(+Boolean)
+%     Can be used if the installation source is a local directory
+%     and the file system supports symbolic links.  In this case
+%     the system adds the current directory to the pack registration
+%     using a symbolic link and performs the local installation steps.
 %
 %   Non-interactive installation can be established using the option
 %   interactive(false). It is adviced to   install from a particular
@@ -679,8 +718,9 @@ pack_create_install_dir(_, _, _) :-
 %     an error and fail.
 
 pack_install(Name, _, Options) :-
-    current_pack(Name),
+    current_pack(Name, Dir),
     option(upgrade(false), Options, false),
+    \+ pack_is_in_local_dir(Name, Dir, Options),
     print_message(error, pack(already_installed(Name))),
     pack_info(Name),
     print_message(information, pack(remove_with(Name))),
@@ -708,8 +748,17 @@ pack_install_from_local(Source, PackTopDir, Name, Options) :-
     exists_directory(Source),
     !,
     directory_file_path(PackTopDir, Name, PackDir),
-    prepare_pack_dir(PackDir, Options),
-    copy_directory(Source, PackDir),
+    (   option(link(true), Options)
+    ->  (   same_file(Source, PackDir)
+        ->  true
+        ;   atom_concat(PackTopDir, '/', PackTopDirS),
+            relative_file_name(Source, PackTopDirS, RelPath),
+            link_file(RelPath, PackDir, symbolic),
+            assertion(same_file(Source, PackDir))
+        )
+    ;   prepare_pack_dir(PackDir, Options),
+        copy_directory(Source, PackDir)
+    ),
     pack_post_install(Name, PackDir, Options).
 pack_install_from_local(Source, PackTopDir, Name, Options) :-
     exists_file(Source),
@@ -718,6 +767,11 @@ pack_install_from_local(Source, PackTopDir, Name, Options) :-
     pack_unpack(Source, PackDir, Name, Options),
     pack_post_install(Name, PackDir, Options).
 
+pack_is_in_local_dir(_Pack, PackDir, Options) :-
+    option(url(DirURL), Options),
+    uri_file_name(DirURL, Dir),
+    same_file(PackDir, Dir).
+
 
 %!  pack_unpack(+SourceFile, +PackDir, +Pack, +Options)
 %
@@ -863,19 +917,24 @@ must_match(Values, Field) :-
 
 %!  prepare_pack_dir(+Dir, +Options)
 %
-%   Prepare for installing the package into  Dir. This should create
-%   Dir if it does not  exist  and   warn  if  the directory already
-%   exists, asking to make it empty.
+%   Prepare for installing the package into  Dir. This
+%
+%     - If the directory exist and is empty, done.
+%     - Else if the directory exists, remove the directory and recreate
+%       it. Note that if the directory is a symlink this just deletes
+%       the link.
+%     - Else create the directory.
 
 prepare_pack_dir(Dir, Options) :-
     exists_directory(Dir),
     !,
     (   empty_directory(Dir)
     ->  true
-    ;   option(upgrade(true), Options)
-    ->  delete_directory_contents(Dir)
-    ;   confirm(remove_existing_pack(Dir), yes, Options),
-        delete_directory_contents(Dir)
+    ;   (   option(upgrade(true), Options)
+        ;   confirm(remove_existing_pack(Dir), yes, Options)
+        )
+    ->  delete_directory_and_contents(Dir),
+        make_directory(Dir)
     ).
 prepare_pack_dir(Dir, _) :-
     make_directory(Dir).
@@ -1010,15 +1069,12 @@ download_scheme(https) :-
 %
 %   Process post installation work.  Steps:
 %
-%     - Create foreign resources [TBD]
+%     - Create foreign resources
 %     - Register directory as autoload library
 %     - Attach the package
 
 pack_post_install(Pack, PackDir, Options) :-
-    post_install_foreign(Pack, PackDir,
-                         [ build_foreign(if_absent)
-                         | Options
-                         ]),
+    post_install_foreign(Pack, PackDir, Options),
     post_install_autoload(PackDir, Options),
     '$pack_attach'(PackDir).
 
@@ -1027,14 +1083,21 @@ pack_post_install(Pack, PackDir, Options) :-
 %   Rebuilt possible foreign components of Pack.
 
 pack_rebuild(Pack) :-
-    '$pack':pack(Pack, BaseDir),
+    current_pack(Pack, PackDir),
     !,
-    catch(pack_make(BaseDir, [distclean], []), E,
-          print_message(warning, E)),
-    post_install_foreign(Pack, BaseDir, []).
+    pack_clean(Pack, PackDir, distclean),
+    post_install_foreign(Pack, PackDir, []).
 pack_rebuild(Pack) :-
     existence_error(pack, Pack).
 
+pack_clean(Pack, PackDir, How) :-
+    (   var(PackDir)
+    ->  current_pack(Pack, PackDir)
+    ;   true
+    ),
+    catch(pack_make(PackDir, [How], []), E,
+          print_message(warning, E)).
+
 %!  pack_rebuild is det.
 %
 %   Rebuild foreign components of all packages.
@@ -1053,10 +1116,15 @@ pack_rebuild :-
 post_install_foreign(Pack, PackDir, Options) :-
     is_foreign_pack(PackDir),
     !,
-    (   option(build_foreign(if_absent), Options),
+    option(rebuild(Rebuild), Options, if_absent),
+    (   Rebuild == if_absent,
         foreign_present(PackDir)
     ->  print_message(informational, pack(kept_foreign(Pack)))
-    ;   setup_path,
+    ;   (   Rebuild == true
+        ->  pack_clean(Pack, PackDir, distclean)
+        ;   true
+        ),
+        setup_path,
         save_build_environment(PackDir),
         configure_foreign(PackDir, Options),
         make_foreign(PackDir, Options)
@@ -1105,12 +1173,17 @@ configure_foreign(PackDir, Options) :-
     exists_file(Configure),
     !,
     build_environment(BuildEnv),
-    run_process(path(bash), [Configure],
+    findall(Opt, configure_option(Opt), Opts),
+    run_process(path(bash), [Configure|Opts],
                 [ env(BuildEnv),
                   directory(PackDir)
                 ]).
 configure_foreign(_, _).
 
+configure_option(Opt) :-
+    prolog_prefix(Prefix),
+    format(atom(Opt), '--prefix=~w', [Prefix]).
+
 make_configure(PackDir, _Options) :-
     directory_file_path(PackDir, 'configure', Configure),
     exists_file(Configure),
@@ -1129,16 +1202,26 @@ autoconf_master('configure.in').
 
 %!  cmake_configure_foreign(+PackDir, +Options) is det.
 %
-%   Create a `build` directory in PackDir and run `cmake ..`
+%   Create a `build` directory in PackDir and run `cmake [options] ..`
 
 cmake_configure_foreign(PackDir, _Options) :-
     directory_file_path(PackDir, build, BuildDir),
     make_directory_path(BuildDir),
-    current_prolog_flag(executable, Exe),
-    format(atom(CDEF), '-DSWIPL=~w', [Exe]),
-    run_process(path(cmake), [CDEF, '..'],
-                [directory(BuildDir)]).
+    findall(Opt, cmake_option(Opt), Argv, [..]),
+    build_environment(BuildEnv),
+    run_process(path(cmake), Argv,
+                [directory(BuildDir),
+                 env(BuildEnv)]).
 
+cmake_option(CDEF) :-
+    current_prolog_flag(executable, Exe),
+    format(atom(CDEF), '-DSWIPL=~w', [Exe]).
+cmake_option(CDEF) :-
+    prolog_prefix(Prefix),
+    format(atom(CDEF), '-DCMAKE_INSTALL_PREFIX=~w', [Prefix]).
+cmake_option(Opt) :-
+    has_program(path(ninja), _),
+    member(Opt, ['-G', 'Ninja']).
 
 %!  make_foreign(+PackDir, +Options) is det.
 %
@@ -1147,14 +1230,6 @@ cmake_configure_foreign(PackDir, _Options) :-
 make_foreign(PackDir, Options) :-
     pack_make(PackDir, [all, check, install], Options).
 
-pack_make(PackDir, Targets, _Options) :-
-    directory_file_path(PackDir, 'Makefile', Makefile),
-    exists_file(Makefile),
-    !,
-    build_environment(BuildEnv),
-    ProcessOptions = [ directory(PackDir), env(BuildEnv) ],
-    forall(member(Target, Targets),
-           run_process(path(make), [Target], ProcessOptions)).
 pack_make(PackDir, Targets, _Options) :-
     directory_file_path(PackDir, 'CMakeLists.txt', CMakefile),
     exists_file(CMakefile),
@@ -1168,8 +1243,21 @@ pack_make(PackDir, Targets, _Options) :-
         forall(member(Target, Targets),
                run_cmake_target(Target, BuildDir, ProcessOptions))
     ).
+pack_make(PackDir, Targets, _Options) :-
+    directory_file_path(PackDir, 'Makefile', Makefile),
+    exists_file(Makefile),
+    !,
+    build_environment(BuildEnv),
+    ProcessOptions = [ directory(PackDir), env(BuildEnv) ],
+    forall(member(Target, Targets),
+           run_process(path(make), [Target], ProcessOptions)).
 pack_make(_, _, _).
 
+%!  run_cmake_target(+Target, +BuildDir, +ProcessOptions) is det.
+%
+%   Run a process step for CMake. We handle `check` using `ctest`, `all`
+%   using ``cmake --build .`` and `install` using ``cmake --install .``.
+
 run_cmake_target(check, BuildDir, ProcessOptions) :-
     !,
     (   directory_file_path(BuildDir, 'CTestTestfile.cmake', TestFile),
@@ -1177,6 +1265,17 @@ run_cmake_target(check, BuildDir, ProcessOptions) :-
     ->  run_process(path(ctest), [], ProcessOptions)
     ;   true
     ).
+run_cmake_target(all, _BuildDir, ProcessOptions) :-
+    !,
+    run_process(path(cmake), ['--build', '.'], ProcessOptions).
+run_cmake_target(install, _BuildDir, ProcessOptions) :-
+    !,
+    run_process(path(cmake), ['--install', '.'], ProcessOptions).
+run_cmake_target(Target, BuildDir, ProcessOptions) :-
+    directory_file_path(BuildDir, 'build.ninja', NinjaFile),
+    exists_file(NinjaFile),
+    !,
+    run_process(path(ninja), [Target], ProcessOptions).
 run_cmake_target(Target, _, ProcessOptions) :-
     run_process(path(make), [Target], ProcessOptions).
 
@@ -1401,6 +1500,43 @@ mingw_root(MinGwRoot) :-
     exists_directory(MinGwRoot),
     !.
 
+%!  prolog_prefix(-Prefix) is semidet.
+%
+%   Return the directory that can be  passed into `configure` or `cmake`
+%   to install executables and other  related   resources  in  a similar
+%   location as SWI-Prolog itself.  Tries these rules:
+%
+%     1. If the Prolog flag `pack_prefix` at a writable directory, use
+%        this.
+%     2. If the current executable can be found on $PATH and the parent
+%        of the directory of the executable is writable, use this.
+%     3. If the user has a writable ``~/bin`` directory, use ``~``.
+
+prolog_prefix(Prefix) :-
+    current_prolog_flag(pack_prefix, Prefix),
+    access_file(Prefix, write),
+    !.
+prolog_prefix(Prefix) :-
+    current_prolog_flag(os_argv, [Name|_]),
+    has_program(path(Name), EXE),
+    file_directory_name(EXE, Bin),
+    file_directory_name(Bin, Prefix0),
+    (   local_prefix(Prefix0, Prefix1)
+    ->  Prefix = Prefix1
+    ;   Prefix = Prefix0
+    ),
+    access_file(Prefix, write),
+    !.
+prolog_prefix(Prefix) :-
+    expand_file_name(~, [UserHome]),
+    directory_file_path(UserHome, bin, BinDir),
+    exists_directory(BinDir),
+    access_file(BinDir, write),
+    !,
+    Prefix = UserHome.
+
+local_prefix('/usr', '/usr/local').
+
 
                  /*******************************
                  *           AUTOLOAD           *
