@@ -41,7 +41,8 @@
 :- use_module(library(debug)).
 
 :- multifile
-    blob_rendering//3.              % +Type, +Blob, +Options
+    blob_rendering//3,              % +Type, +Blob, +Options
+    portray//2.                     % +Term, +Options
 
 /** <module> Represent Prolog terms as HTML
 
@@ -56,9 +57,12 @@ terms as structured HTML.
 %   passed to write_term/3. In addition,   the following options are
 %   processed:
 %
+%     - format(+Format)
+%     Used for atomic values.  Typically this is used to
+%     render a single value.
 %     - float_format(+Format)
 %     If a float is rendered, it is rendered using
-%     `format(string(S), Format, [Float])`%
+%     `format(string(S), Format, [Float])`
 %
 %   @tbd    Cyclic terms.
 %   @tbd    Attributed terms.
@@ -74,7 +78,7 @@ term(Term, Options) -->
                       depth(0)
                     ],
                     Options1),
-      dict_create(Dict, _, Options1)
+      dict_options(Dict, Options1)
     },
     any(Term, Dict).
 
@@ -83,6 +87,13 @@ any(_, Options) -->
     { Options.depth >= Options.max_depth },
     !,
     html(span(class('pl-ellipsis'), ...)).
+any(Term, Options) -->
+    (   {   nonvar(Term)
+        ;   attvar(Term)
+        }
+    ->  portray(Term, Options)
+    ),
+    !.
 any(Term, Options) -->
     { primitive(Term, Class0),
       !,
@@ -397,12 +408,12 @@ end_code_type(List, Type, _) :-
     Type = punct.
 end_code_type(OpTerm, Type, Options) :-
     compound_name_arity(OpTerm, Name, 1),
-    is_op1(Name, Type, Pri, ArgPri, Options),
+    is_op1(Name, OpType, Pri, ArgPri, Options),
     \+ Options.get(ignore_ops) == true,
     !,
     (   Pri > Options.priority
     ->  Type = punct
-    ;   (   Type == prefix
+    ;   (   OpType == prefix
         ->  end_code_type(Name, Type, Options)
         ;   arg(1, OpTerm, Arg),
             arg_options(Options, ArgOptions),
@@ -424,10 +435,14 @@ end_code_type(Compound, Type, Options) :-
     compound_name_arity(Compound, Name, _),
     end_code_type(Name, Type, Options).
 
-end_type(S, Type, _Options) :-
+end_type(S, Type, Options) :-
     number(S),
     !,
-    Type = alnum.
+    (   (S < 0 ; S == -0.0),
+        Options.side == left
+    ->  Type = symbol
+    ;   Type = alnum
+    ).
 end_type(S, Type, Options) :-
     Options.side == left,
     !,
@@ -499,6 +514,17 @@ quote_atomic(Float, String, Options) :-
     Format = Options.get(float_format),
     !,
     format(string(String), Format, [Float]).
+quote_atomic(Plain, String, Options) :-
+    atomic(Plain),
+    Format = Options.get(format),
+    !,
+    format(string(String), Format, [Plain]).
+quote_atomic(Plain, String, Options) :-
+    rational(Plain),
+    \+ integer(Plain),
+    !,
+    operator_module(Module, Options),
+    format(string(String), '~W', [Plain, [module(Module)]]).
 quote_atomic(Plain, Plain, _) :-
     number(Plain),
     !.
