@@ -3633,24 +3633,24 @@ Sread_memfile(void *handle, char *buf, size_t size)
 }
 
 
-static long
-Sseek_memfile(void *handle, long offset, int whence)
+static int64_t
+Sseek_memfile64(void *handle, int64_t offset, int whence)
 { memfile *mf = handle;
 
   switch(whence)
   { case SIO_SEEK_SET:
       break;
     case SIO_SEEK_CUR:
-      offset += (long)mf->here;		/* Win64: truncates */
+      offset += mf->here;
       break;
     case SIO_SEEK_END:
-      offset = (long)mf->size - offset;	/* Win64 */
+      offset = mf->size - offset;
       break;
     default:
       errno = EINVAL;
       return -1;
   }
-  if ( offset < 0 || offset > (long)mf->size )
+  if ( offset < 0 || offset > mf->size )
   { errno = EINVAL;
     return -1;
   }
@@ -3660,6 +3660,24 @@ Sseek_memfile(void *handle, long offset, int whence)
 }
 
 
+static long
+Sseek_memfile(void *handle, long offset, int whence)
+{ memfile *mf = handle;
+  size_t old = mf->here;
+
+  int64_t rval = Sseek_memfile64(handle, (int64_t)offset, whence);
+  long rc = (long)rval;
+
+  if ( rval == rc )
+  { return rc;
+  } else
+  { mf->here = old;
+    errno = EINVAL;
+    return -1;
+  }
+}
+
+
 static int
 Sclose_memfile(void *handle)
 { memfile *mf = handle;
@@ -3691,6 +3709,12 @@ Scontrol_memfile(void *handle, int action, void *arg)
       *szp = mf->here;
       return 0;
     }
+    case SIO_GETREPOSITION:
+    { int *valp = arg;
+      *valp = TRUE;
+
+      return 0;
+    }
     default:
       errno = EINVAL;
       return -1;
@@ -3703,7 +3727,8 @@ IOFUNCTIONS Smemfunctions =
   Swrite_memfile,
   Sseek_memfile,
   Sclose_memfile,
-  Scontrol_memfile
+  Scontrol_memfile,
+  Sseek_memfile64
 };
 
 
