@@ -146,6 +146,7 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(MSG_SHIFT_FRAME),
   DEBUG_TOPIC(MSG_STACK_OVERFLOW),
   DEBUG_TOPIC(MSG_UNWIND_EXCEPTION),
+  DEBUG_TOPIC(MSG_AGC_CONSIDER),
 
   DEBUG_TOPIC(MSG_ATTVAR_LINK),
   DEBUG_TOPIC(MSG_CALL_RESIDUE_VARS),
@@ -212,6 +213,7 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(MSG_TABLING_RESTRAINT),
   DEBUG_TOPIC(MSG_TABLING_MONOTONIC),
   DEBUG_TOPIC(MSG_TABLING_TRANSACTION),
+  DEBUG_TOPIC(MSG_TABLING_IDG_REEVAL_NODE),
 
   DEBUG_TOPIC(TABLING_NO_EARLY_COMPLETION),
   DEBUG_TOPIC(TABLING_NO_SIMPLIFY),
