@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1997-2020, University of Amsterdam
+    Copyright (c)  1997-2021, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -110,7 +111,6 @@ struct PL_global_data
 
   struct
   { sig_handler handlers[MAXSIGNAL];	/* How Prolog preceives signals */
-#ifdef HAVE_SIGNAL
     int		sig_alert;		/* our alert signal */
 #endif
   } signals;
@@ -280,6 +280,7 @@ struct PL_global_data
       struct event_list *onbreak;	/* breakpoint change */
       struct event_list *onframefinish; /* Debugged frame finished */
 #ifdef O_PLMT
+      struct event_list *onthreadstart;	/* thread start hook */
       struct event_list *onthreadexit;	/* thread exit hook */
 #endif
       struct event_list *onuntable;	/* Untable after reload */
@@ -455,6 +456,12 @@ struct PL_local_data
     int		is_sync;		/* current signal is synchronous */
 #ifndef __unix__
     int		forced;			/* Forced signal */
+#endif
+#ifdef HAVE_SIGNAL
+#ifdef O_C_STACK_GUARDED
+    jmp_buf	context;		/* Guarded for C-stack overflows */
+    int		sig_critical;		/* context is valid */
+    void       *alt_stack;		/* The stack */
 #endif
   } signal;
 
