@@ -65,6 +65,7 @@ option  parsing,  initialisation  and  handling  of errors and warnings.
 #include "os/pl-prologflag.h"
 #include "os/pl-ctype.h"
 #include "os/pl-utf8.h"
+#include "os/pl-cstack.h"
 #include <errno.h>
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
@@ -1337,7 +1338,8 @@ version(void)
 
 static int
 abi_version(void)
-{ setupProlog();
+{ initDefaultOptions();
+  setupProlog();
   Sprintf(PLPKGNAME "-abi-%d-%d-%08x-%08x\n",
 	  PL_FLI_VERSION,
 	  PL_QLF_LOADVERSION,
@@ -1446,9 +1448,7 @@ with -DGC_DEBUG.
 
 int
 cleanupProlog(int rval, int reclaim_memory)
-{ GET_LD
-
-  if ( GD->cleaning != CLN_NORMAL )
+{ if ( GD->cleaning != CLN_NORMAL )
     return FALSE;
 
 #ifdef __WINDOWS__
@@ -1463,12 +1463,14 @@ cleanupProlog(int rval, int reclaim_memory)
   }
 
 #ifdef O_PLMT
-  if ( !LD )
+  if ( !GLOBAL_LD )
   { PL_thread_attach_engine(NULL);
-    LD = GLOBAL_LD;
-    if ( !LD )
-      goto emergency;
   }
+  GET_LD
+  if ( !LD )
+    goto emergency;
+#else
+  GET_LD
 #endif
 
   GD->cleaning = CLN_PROLOG;
@@ -1506,7 +1508,8 @@ cleanupProlog(int rval, int reclaim_memory)
   resetProfiler();			/* don't do profiling anymore */
 #endif
 #ifdef O_PLMT
-  exitPrologThreads();
+  if ( !exitPrologThreads() )		/* reclaim memory while a thread */
+    reclaim_memory = FALSE;		/* runs is likely to crash */
 
 emergency:
 #endif
@@ -1544,7 +1547,7 @@ emergency:
 #endif
 
   if ( reclaim_memory )
-  { freeStacks(PASS_LD1);
+  { freeStacks();
 #ifdef O_PLMT
     cleanupLocalDefinitions(LD);
 #endif
@@ -1673,14 +1676,20 @@ vsysError(const char *fm, va_list args)
     unblockSignals(&LD->gc.saved_sigmask);
   }
 
+  Sfprintf(Serror, "\n");
+  save_backtrace("SYSERROR");
+  print_backtrace_named("SYSERROR");
+
 #if defined(O_DEBUGGER)
   Sfprintf(Serror, "\n\nPROLOG STACK:\n");
   PL_backtrace(10, 0);
   Sfprintf(Serror, "]\n");
 #endif /*O_DEBUGGER*/
 
-  if ( GD->bootsession )
-    PL_abort_process();
+  if ( !(true(Sinput, SIO_ISATTY) &&
+	 true(Serror, SIO_ISATTY)) ||
+       GD->bootsession )
+    PL_abort_process();			/* non-interactive or booting */
 
 action:
 #ifdef HAVE_GETPID
