@@ -302,18 +302,7 @@ All  thread-local  data  is  combined  in    one  structure  defined  in
 pl-global.h. If Prolog is compiled for single-threading this is a simple
 global variable and the macro LD is defined   to  pick up the address of
 this variable. In multithreaded context,  POSIX pthread_getspecific() is
-used to get separate versions for each  thread. Functions uisng LD often
-may wish to write:
-
-<header>
-{ GET_LD
-#undef LD
-#define LD LOCAL_LD
-  ...
-
-#undef LD
-#define LD GLOBAL_LD
-}
+used to get separate versions for each thread.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 		 /*******************************
@@ -430,7 +419,6 @@ bool		aliasThread(int tid, atom_t type, atom_t name);
 word		pl_thread_create(term_t goal, term_t id,
 				 term_t options);
 word		pl_thread_exit(term_t retcode);
-foreign_t	pl_thread_signal(term_t thread, term_t goal);
 
 foreign_t	pl_thread_at_exit(term_t goal);
 int		PL_thread_self(void);
@@ -454,7 +442,6 @@ int		PL_mutex_unlock(struct pl_mutex *m);
 int		PL_thread_raise(int tid, int sig);
 void		cleanupThreads(void);
 intptr_t	system_thread_id(PL_thread_info_t *info);
-double	        ThreadCPUTime(PL_local_data_t *ld, int which);
 void		get_current_timespec(struct timespec *time);
 void	        carry_timespec_nanos(struct timespec *time);
 int		signal_waiting_threads(Module m, thread_wait_channel *wch);
@@ -472,8 +459,10 @@ void	resumeThreads(void);
 void	markAtomsMessageQueues(void);
 void	markAtomsThreadMessageQueue(PL_local_data_t *ld);
 
-#define acquire_ldata(info)	acquire_ldata__LD(info PASS_LD)
+#define acquire_ldata(info)	LDFUNC(acquire_ldata, info)
 #define release_ldata(ld)	(LD->thread.info->access.ldata = NULL)
+/* defined in pl-inline.h */
+static inline PL_local_data_t *acquire_ldata(DECL_LD PL_thread_info_t *info);
 
 		 /*******************************
 		 *     CONDITION VARIABLES	*
@@ -543,8 +532,6 @@ int	cv_timedwait(message_queue *queue,
 #define acquire_ldata(ld)	(ld)
 #define release_ldata(ld)	(void)0
 
-double	        ThreadCPUTime(PL_local_data_t *ld, int which);
-
 #endif /*O_PLMT*/
 
 		 /*******************************
@@ -558,9 +545,24 @@ foreign_t	pl_thread_self(term_t self);
 
 typedef struct
 { functor_t functor;			/* functor of property */
+  /* FIXME: we should not be storing function pointers with
+   * incomplete prototypes! WASM in particular cannot call a
+   * function unless it has a full, correct prototype
+   */
   int (*function)();			/* function to generate */
 } tprop;
 
+#if USE_LD_MACROS
+#define	pushPredicateAccessObj(def)	LDFUNC(pushPredicateAccessObj, def)
+#define	popPredicateAccess(def)		LDFUNC(popPredicateAccess, def)
+#define	popNPredicateAccess(n)		LDFUNC(popNPredicateAccess, n)
+#define	cgc_thread_stats(stats)		LDFUNC(cgc_thread_stats, stats)
+#define	isSignalledGCThread(sig)	LDFUNC(isSignalledGCThread, sig)
+#define	ThreadCPUTime(which)		LDFUNC(ThreadCPUTime, which)
+#endif /*USE_LD_MACROS*/
+
+#define LDFUNC_DECLARATIONS
+
 int		get_prop_def(term_t t, atom_t expected,
 			     const tprop *list, const tprop **def);
 void		initPrologThreads(void);
@@ -570,19 +572,15 @@ Atom**		pl_atom_buckets_in_use(void);
 Definition*	predicates_in_use(void);
 int		pl_functor_table_in_use(FunctorTable functor_table);
 int		pl_kvs_in_use(KVS kvs);
-definition_ref* pushPredicateAccessObj(Definition def ARG_LD);
-void		popPredicateAccess__LD(Definition def ARG_LD);
-size_t		popNPredicateAccess__LD(size_t n ARG_LD);
+definition_ref* pushPredicateAccessObj(Definition def);
+void		popPredicateAccess(Definition def);
+size_t		popNPredicateAccess(size_t n);
 void		markAccessedPredicates(PL_local_data_t *ld);
-int		cgc_thread_stats(cgc_stats *stats ARG_LD);
+int		cgc_thread_stats(cgc_stats *stats);
 int		signalGCThread(int sig);
-int		isSignalledGCThread(int sig ARG_LD);
+int		isSignalledGCThread(int sig);
+double	        ThreadCPUTime(int which);
 
-		 /*******************************
-		 *	LD-USING FUNCTIONS	*
-		 *******************************/
-
-#define popPredicateAccess(def) popPredicateAccess__LD(def PASS_LD)
-#define popNPredicateAccess(cnt) popNPredicateAccess__LD(cnt PASS_LD)
+#undef LDFUNC_DECLARATIONS
 
 #endif /*PL_THREAD_H_DEFINED*/
