@@ -495,13 +495,23 @@ PL_discard_foreign_frame(fid_t id)
 }
 
 
-#define determinism_error(fr, found) LDFUNC(determinism_error, fr, found)
+#define determinism_error(fr, bfr, found) \
+	LDFUNC(determinism_error, fr, bfr, found)
+
 static int
-determinism_error(DECL_LD LocalFrame fr, atom_t found)
+determinism_error(DECL_LD LocalFrame fr, Choice bfr, atom_t found)
 { fid_t fid;
   int rc = FALSE;
   atom_t a = ATOM_error;
 
+  if ( found == ATOM_nondet )
+  { for(; bfr && bfr->type == CHP_DEBUG; bfr=bfr->parent)
+      ;
+
+    if ( (void*)bfr < (void*)fr )
+      return TRUE;
+  }
+
   PL_current_prolog_flag(ATOM_determinism_error, PL_ATOM, &a);
   if ( a == ATOM_silent )
     return TRUE;
@@ -573,7 +583,7 @@ ssu_or_det_failed(DECL_LD LocalFrame fr)
   int rc = FALSE;
 
   if ( false(fr, FR_SSU_DET) )
-    return determinism_error(fr, ATOM_fail);
+    return determinism_error(fr, NULL, ATOM_fail);
 
   if ( (fid = PL_open_foreign_frame()) )
   { term_t goal;
@@ -2111,7 +2121,7 @@ exception_hook(DECL_LD qid_t pqid, term_t fr, term_t catchfr_ref)
       PL_put_frame(av+2, (LocalFrame)valTermRef(fr));
 
       if ( !catchfr_ref )
-	catchfr_ref = isCaughtInOuterQuery(pqid, exception_term);
+	catchfr_ref = isCaughtInOuterQuery(pqid, exception_bin);
       if ( catchfr_ref == (term_t)-1 )
       { PL_put_atom_chars(av+3, "C");
       } else if ( catchfr_ref && catchfr_ref == LD->exception.fr_rewritten )
@@ -2129,9 +2139,11 @@ exception_hook(DECL_LD qid_t pqid, term_t fr, term_t catchfr_ref)
       { PL_put_frame(av+3, NULL);	/* puts 'none' */
       }
 
+      startCritical;
       qid = PL_open_query(MODULE_user, PL_Q_NODEBUG|PL_Q_CATCH_EXCEPTION,
 			  PROCEDURE_exception_hook4, av);
       rc = PL_next_solution(qid);
+      rc = endCritical && rc;
       debug = debugstatus.debugging;
       trace = debugstatus.tracing;
       if ( rc )				/* pass user setting trace/debug */
@@ -2901,8 +2913,20 @@ struct register_file;
 #  define VMI_ARG_DECL Code PC
 #  define VMI_ARG_PASS PC
 # else
-#  define VMI_ARG_DECL Code PC, struct register_file *registers ARG_LD
-#  define VMI_ARG_PASS PC, registers PASS_LD
+/* Avoiding the LDFUNC builtins entirely here, because they don't play
+ * nicely with indirect usage, and the VMI-function/non-VMI-function
+ * switch is already complicated enough. Luckily, the VMI functions
+ * are entirely self-contained and they are only ever called from our
+ * macros (and from functions with local __PL_ld defined), so replicating
+ * the pl-builtin.h logic here isn't such a hazard.
+ */
+#  if (defined(O_PLMT) || defined(O_MULTIPLE_ENGINES)) && USE_LD_MACROS
+#   define VMI_ARG_DECL PL_local_data_t *__PL_ld, Code PC, struct register_file *registers
+#   define VMI_ARG_PASS __PL_ld, PC, registers
+#  else
+#   define VMI_ARG_DECL Code PC, struct register_file *registers
+#   define VMI_ARG_PASS PC, registers
+#  endif
 # endif
 typedef VMI_RETTYPE (*vmi_instr)(VMI_ARG_DECL);
 # if VMI_REGISTER_VARIABLES
