@@ -44,6 +44,7 @@
 #include "pl-wam.h"
 #include "pl-prims.h"
 #include "pl-modul.h"
+#include "pl-setup.h"
 #include <math.h>
 #include "os/pl-dtoa.h"
 #include "os/pl-ctype.h"
@@ -181,7 +182,7 @@ writeNumberVar(DECL_LD term_t t, write_options *options)
 
   p = &f->arguments[0];
   deRef(p);
-  if ( isInteger(*p) )
+  if ( isTaggedInt(*p) || isBignum(*p) )
   { int64_t n = valInteger(*p);
     char buf[32];			/* Max is H354745078340568300 */
 
@@ -1764,7 +1765,10 @@ writeTopTerm(term_t term, int prec, write_options *options)
   Slock(options->out);
   if ( (!(options->flags&PL_WRT_NO_CYCLES) && options->max_depth) ||
        PL_is_acyclic(term) )
-  { rc = writeTerm(term, prec, options);
+  { C_STACK_OVERFLOW_GUARDED(
+	rc,
+	writeTerm(term, prec, options),
+        (void)0);
   } else
   { fid_t fid;
     term_t template, substitutions, cycles, at_term;
@@ -1784,7 +1788,10 @@ writeTopTerm(term_t term, int prec, write_options *options)
 			  PL_TERM, template,
 			  PL_TERM, cycles) )
       return FALSE;
-    rc = writeTerm(at_term, prec, options);
+    C_STACK_OVERFLOW_GUARDED(
+	rc,
+	writeTerm(at_term, prec, options),
+	(void)0);
     PL_discard_foreign_frame(fid);
   }
   Sunlock(options->out);
